#include <algorithm>
#include <fstream>
#include <iostream>
#include <iterator>
#include <ostream>
#include <regex.h>
#include <string>
#include <vector>

#include "io.hpp"
#include "helpers.hpp"

using namespace std;

void read_genomes(const string &fname, vector<genome> &genomes, vector<string> &names, int circular,
		vector<string> &nmap, bool allowdel, bool allowdup, int normto){

	bool err = false;						// error flag
	ifstream taxafile;						// input file handle
	int s;									// sign
	map<string, int> nm,					// mapping from genenames to integers
		cnt;								// number of occ. of genes in one genome
	string line,							// one line of the text
		g;
	unsigned n = 1;							// element number
	vector<int> chr;						// one chromosome
	vector<string> tg;						// one text genome
	vector<vector<string> > txtgenomes;		// all the textgenomes from the file
	vector<vector<int> > signs;

	// open file
	taxafile.open(fname.c_str(), ios::in);
	if(!taxafile.is_open()){
		cerr << "read_genomes: could not read: "<< fname.c_str()<<endl;
		exit(1);
	}

	// recycle existing name map
	//for(unsigned i=1; i<nmap.size(); i++){
	//		nm[ nmap[i] ] = i;
	//}
	//n = nmap.size()+1;

	// read the file line for line
	// and save the names of the genomes as well as the genomes as text
	while(taxafile){

		getline(taxafile, line);

		if (!taxafile)
			break;

		strip_leading_ws(line);
		strip_trailing_ws(line);
			// skip empty or comment lines as well as lines containing trees
		if(line.size() == 0 || line[0] == '#' || line[0] == '(' || line[0] == ']')
			continue;

			// get the species name from 'name lines'
			// and start a new genome
		if(line[0] == '>'){
			line = line.erase(line.find_last_not_of(" ") + 1 ) ;
			names.push_back( line.substr(1,line.length()) );
			txtgenomes.push_back( vector<string>() );
			signs.push_back( vector<int>() );
		}
			// and the genomes from the other lines
			// add them to the last added genome (so multiline fasta should be possible)
		else{
			if( signs.size() == 0 ){
				cerr << "error: missing header"<<endl;
				exit(EXIT_FAILURE);
			}

			split_string(line, tg);
			for(unsigned i=0; i<tg.size(); i++){
				if( tg[i][0] == '-' ){
					s = -1;
					g = tg[i].substr(1);
				}else if( tg[i][0] == '+' ){
					s = 1;
					g = tg[i].substr(1);
				}else{
					s = 1;
					g = tg[i];
				}
				signs.back().push_back( s );
				txtgenomes.back().push_back( g );

				if( nm.find(g) == nm.end() ){
					nm[ g ] = n;
					n++;
				}
			}
			tg.clear();
		}
	}
	taxafile.close();

	// check if the number of name lines is equal to the number of genome lines
	if(names.size() != txtgenomes.size()){
		cout << "unequal number of names and genomes in the file"<<endl;
		cout << "names "<<endl;
		for(unsigned i=0; i<names.size(); i++){
		 	cout << i<< " : "<<names[i]<<endl;
		}
		cout << "genomes "<<endl;
		for(unsigned i=0; i<txtgenomes.size(); i++){
			cout << i<< " : "; copy(txtgenomes[i].begin(), txtgenomes[i].end(), ostream_iterator<string>(cout," ")); cout << endl;
		}
		exit(1);
	}

		// init the gene occurence counter map (needed for checking for
		// duplications and deletions)
	for( map<string,int>::iterator it=nm.begin(); it!=nm.end(); it++ ){
		cnt[ it->first ] = 0;
	}

	nmap.resize(n);
	nmap[0] = "INVALID_GENE_0";
	for( map<string,int>::iterator it=nm.begin(); it!=nm.end(); it++ ){
		nmap[it->second] = it->first;
	}

	for(unsigned i=0; i<txtgenomes.size(); i++){
		for(unsigned j=0; j<txtgenomes[i].size(); j++){
			chr.push_back( signs[i][j] * nm[ txtgenomes[i][j] ] );
			cnt[ txtgenomes[i][j] ]++;
		}
		genomes.push_back( genome(chr, circular, &nmap, normto) );
		for( map<string, int>::iterator it = cnt.begin(); it!=cnt.end(); it++ ){
			if( allowdel == false && it->second == 0 ){
				cerr << names[i]<<": missing gene "<< it->first <<endl;
				err = true;
			}else if( allowdup == false && it->second > 1 ){
				cerr <<i<< names[i]<<": "<<it->second<<" gene "<<it->first <<endl;
				err = true;
			}
			cnt[it->first] = 0;
		}
		chr.clear();
	}
	cnt.clear();

	if( err ){
		cout << "errors occured while reading "<< fname << " -> exiting"<<endl;
		exit(1);
	}
		// strip leading and trailing white spaces from the species names
	for(unsigned i=0; i<names.size(); i++){
		strip_leading_ws(names[i]);
		strip_trailing_ws(names[i]);
	}

	txtgenomes.clear();
}
