#include <fstream>
#include <vector>

#ifndef _IO_HPP_
#define _IO_HPP_

#include "genome.hpp"


/**
 * get program parameters
 * @param[in] argc argument count
 * @param[in] argv arguments
 * @param[out] fname filename
 */
void getoptions(int argc, char *argv[], string &fname);

/**
 * read gene orders from a fasta like file
 * @param[in] fname the name of the file to read
 * @param[out] genomes the genomes as permutations
 * @param[out] names the names of the genomes as given in the file
 * @param[in] circular handle genomes as circular, i.e. shift and rotate them to the same element
 * @param[out] nmap the name map, maps the integers to the names of the genes
 * @param[in] allowdel allow deleted elements
 * @param[in] allowdup allow duplicated elements
 * @param[in] normto rotate circular genomes to the specified integer
 */
void read_genomes(const string &fname, vector<genome> &genomes, vector<string> &names, int circular,
		vector<string> &nmap, bool allowdel=false, bool allowdup=false, int normto=1 );

/**
 * print usage info and exit
 */
void usage();

#endif//_IO_HPP_
